import logging
import requests
import sys
from multipledispatch import dispatch
from pyrate_limiter import Duration, Rate, Limiter, BucketFullException, LimiterDelayException
from url_revive.config import settings

@dispatch(str)
def safe_get(url, params=None):
    default_timeout = (settings.NETWORK.CONNECT_TIMEOUT, settings.NETWORK.READ_TIMEOUT)
    try:
        response = requests.get(url, params=params, timeout=default_timeout)
        response.raise_for_status()
        return response
    except requests.exceptions.HTTPError as e:
        logging.error(f'HTTP Error: {e}')
    except requests.exceptions.ConnectionError:
        logging.error(f'Error connecting to {url}')
    except requests.exceptions.Timeout:
        logging.error(f'Connection to {url} timed out')
    except requests.exceptions.RequestException as e:
        logging.error(f'Unhandled RequestException {e}')

@dispatch(list,int)
def safe_get(urls, throttle_rate):
    rate_limit = Rate(1, Duration.SECOND * throttle_rate)
    limiter = Limiter(rate_limit, max_delay=Duration.MINUTE) 
    for url in urls:
        try:
            limiter.try_acquire(url)  
            yield safe_get(url)
        except BucketFullException as e:
            logging.error(f'Rate limit exceeded for {url}: {e}')
        except LimiterDelayException:
            logging.error(f"Delay exceeded for {url} aborting...")

class RequestThrottler:
    def __init__(self, throttle_rate):
        self.rate_limit = Rate(1, Duration.SECOND * throttle_rate)
        self.limiter = Limiter(self.rate_limit, max_delay=Duration.MINUTE) 

    def safe_get(self, url, params=None):
        try:
            self.limiter.try_acquire(url)  
            return safe_get(url, params=params)
        except BucketFullException as e:
            logging.error(f'Rate limit exceeded for {url}: {e}')
        except LimiterDelayException:
            logging.error(f"Delay exceeded for {url} aborting...")

def main():
    logging.basicConfig(
        level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
    )
    url = sys.argv[1]
    count = int(sys.argv[2])
    urls = [url] * count
    for rsp in safe_get(urls, settings.WAYBACK_API_LIMIT):
        logging.info(rsp)

if __name__=='__main__':
    main()



import argparse
import sys
from argparse import FileType
from urllib.parse import urlparse, urlunparse
from url_revive.config import settings

ascii_art = r"""
 _   _ ___ _      ___         _
| | | | _ \ |    | _ \_____ _(_)_ _____
| |_| |   / |__  |   / -_) V / \ V / -_)
 \___/|_|_\____| |_|_\___|\_/|_|\_/\___|
"""

def get_scheme(url):
    parsed_url = urlparse(url)
    return parsed_url.scheme

def replace_scheme(url, new_scheme='https'):
    parsed_url = urlparse(url)
    if not get_scheme(url):
        return f'{new_scheme}://{url}'
    new_url = parsed_url._replace(scheme=new_scheme)
    return urlunparse(new_url)

def parse_urls(args):
    urls = []
    if args.url:
        urls.append(args.url)
    elif not args.file.isatty():
        urls.extend([line.strip() for line in args.file])
    for i, url in enumerate(urls):
        if not get_scheme(url):
            urls[i] = replace_scheme(urls[i], 'https')
    return urls

def add_input_options(parser):
    input_group = parser.add_argument_group('INPUT')
    input_group.add_argument(
        '-u', '--url', type=str, help='Fetch snapshots for single URL'
    )
    input_group.add_argument(
        '-f',
        '--file',
        type=FileType('r'),
        default='-',
        nargs='?',
        help='Read multiple URLs from file',
    )

def add_filter_options(parser):
    filter_group = parser.add_argument_group('FILTER')
    filter_group.add_argument(
        '-l',
        '--limit',
        type=int,
        default=settings.NETWORK.DEFAULT_FETCH_LIMIT,
        help='Limit number of snapshots fetched per URL',
    )
    filter_group.add_argument(
        '-mc',
        '--match-codes',
        type=str,
        help='Status codes to match -> -mc 200,302,404',
    )
        
def add_general_options(parser):
    general_group = parser.add_argument_group('GENERAL')
    general_group.add_argument(
        '-d',
        '--dump',
        action='store_true',
        help='Print source code dump from snapshots',
    )
    general_group.add_argument(
        '-nc',
        '--no-color',
        action='store_true',
        default=False,
        help='Do not color output',
    )
    general_group.add_argument(
        '-m',
        '--memento',
        action='store_true',
        help='Fetch snapshots from multiple archives with the Memento API',
    )
    general_group.add_argument(
        '-r',
        '--raw',
        action='store_true',
        default=False,
        help='Exclude wayback metadata from snapshots',
    )
    general_group.add_argument(
        '-s',
        '--silent',
        action='store_true',
        help='Only show snapshots or source code dumps',
    )
    general_group.add_argument(
        '--unique',
        action='store_true',
        help='Exclude duplicate URLs',
    )

def parse_args():
    parser = argparse.ArgumentParser()
    add_input_options(parser)
    add_general_options(parser)
    add_filter_options(parser)

    args = parser.parse_args()
    if not args.silent:
        print(ascii_art, file=sys.stderr)
    if args.match_codes:
        args.match_codes = args.match_codes.split(',')
    args.urls = parse_urls(args)
    return args



import colorlog
import logging
from colorama import Fore,Style
from url_revive.archive_reader import fetch_cdx_snapshots, fetch_memento_snapshots
from url_revive.cli_parser import parse_args 
from url_revive.networking import RequestThrottler

def query_wayback(urls, limit, match_codes, raw=False, unique=False):
    for url in urls:
        logging.info(f'Fetching wayback snapshot(s) for {url}')
        snapshots = fetch_cdx_snapshots(url, limit, match_codes, raw=raw, unique=unique)
        for snapshot in snapshots:
            yield snapshot

def query_memento(urls, limit, match_codes):
    for url in urls:
        logging.info(f'Fetching memento snapshot(s) for {url}')
        snapshots = fetch_memento_snapshots(url, limit, match_codes)
        for snapshot in snapshots:
            yield snapshot

def dump_snapshots(snapshots):
    request_throttler = RequestThrottler(1)
    for snapshot in snapshots:
        dump_snapshot(snapshot, request_throttler) 

def dump_snapshot(snapshot, request_throttler):
    url = snapshot.url
    response = request_throttler.safe_get(url)
    if not response:
        logging.error(f'Failed to dump snapshot')
    else:
        print(response.text)

def pretty_print(snapshot, color=True):
    status_code = int(snapshot.status) // 100
    match status_code:
        case 1:
            status_color = Fore.WHITE
        case 2:
            status_color = Fore.GREEN 
        case 3:
            status_color = Fore.YELLOW 
        case 4:
            status_color = Fore.RED
        case 5:
            status_color = Fore.MAGENTA
    if color: 
        status_display = f'{status_color}{snapshot.status}{Style.RESET_ALL}'
    else:
        status_display = f'{snapshot.status}'
    print(f'{snapshot.url} [{status_display}]')

def setup_logger(silent, no_color):
    logging.getLogger("pyrate_limiter").setLevel(logging.CRITICAL)
    logging_level = logging.WARNING if silent else logging.INFO

    logger = logging.getLogger()
    logger.setLevel(logging_level)
    handler = colorlog.StreamHandler()
    formatter = colorlog.ColoredFormatter(
        "[%(log_color)s%(levelname)s%(reset)s] %(blue)s%(message)s",
        datefmt=None,
        reset=True,
        log_colors={
            'DEBUG'    :   'blue',
            'INFO'     :   'green',
            'WARNING'  :   'yellow',
            'ERROR'    :   'red',
            'CRITICAL' :   'bold_red',
        }
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)

def main():
    args = parse_args() 
    setup_logger(args.silent, args.no_color)
    request_throttler = RequestThrottler(4)

    if args.memento:
        queries = query_memento(args.urls, args.limit, args.match_codes)
    else:
        queries = query_wayback(args.urls, args.limit, args.match_codes, raw=args.raw, unique=args.unique)

    for query in queries:
        for snapshot in query:
            if snapshot.status == '-': #exclude continuation
                continue
            pretty_print(snapshot, color= not args.no_color)
            if args.dump:
                dump_snapshot(snapshot, request_throttler)

if __name__ == "__main__":
    main()



import json
import logging
import sys
from dataclasses import dataclass
from enum import Enum
from url_revive.config import settings
from url_revive.networking import RequestThrottler

class ArchiveFormat(Enum):
    CSV = 'csv'
    JSON = 'json'
    JSONL = 'jsonl'
    JSON_CSV = 'json_csv'

@dataclass
class Snapshot:
    url: str
    status: int

def json_csv_to_json(json_array):
    if not json_array:
        return []
    keys = json_array[0]
    rows = json_array[1:]
    return [dict(zip(keys, row)) for row in rows if row]

def format_data_to_json(data, output_format):
    match output_format:
        case ArchiveFormat.CSV:
            data = data.json()
            data = json_csv_to_json(data)
        case ArchiveFormat.JSON:
            data = json.loads(data.text) 
        case ArchiveFormat.JSONL:
            data = data.text.split('\n')
            data = [json.loads(entry) for entry in data if entry]
    return data

def parse_snapshot(record, web_archive, raw=True):
    record_timestamp = record["timestamp"]
    if raw:
        record_timestamp += "id_"
    return f'{web_archive}/{record_timestamp}/{record["original"]}'

def fetch_memento_snapshots(url, limit, match_codes):
    request_throttler = RequestThrottler(1)
    for archive in settings.memento.archives:
        if not archive.ENABLED:
            continue
        logging.info(f'Sending query to archive - {archive.name}')
        memento_url = f'{archive.HOST}/{settings.memento.JSON}/{url}'
        response = request_throttler.safe_get(memento_url)
        output_type = ArchiveFormat(archive.OUTPUT_FORMAT)
        try:
            json_data = format_data_to_json(response, output_type)
        except Exception as e:  
            logging.error(f'Failed to parse response from archive, error - {e}')
            yield []
        snapshots = []
        for entry in json_data:
            timestamp = entry['timestamp']
            snapshot_url = f'{archive.host}/{timestamp}/{url}'
            snapshot = Snapshot(snapshot_url, entry[archive.status_key])
            snapshots.append(snapshot)
        if match_codes:
            snapshots = [snap for snap in snapshots if snap.status in match_codes]
        yield snapshots[:limit]

def fetch_cdx_snapshots(url, limit, match_codes, raw=False, unique=False):
    request_throttler = RequestThrottler(1)
    params = { 'output': 'json', 'url':url, 'limit':limit }
    if unique:
        params['collapse'] = 'urlkey'
    if match_codes is not None:
        mc_key = 'filter'
        mc_value = f'statuscode:({"|".join(match_codes)})'
        params[mc_key] = mc_value
    response = request_throttler.safe_get(
        url=settings.WAYBACK.API_CDX, params=params
    )
    if not response:
        logging.error('Request to Wayback failed')
        return []
    try:
        data = response.json()
        json_data = json_csv_to_json(data)
    except Exception as e:  
        logging.error('Failed to parse response from wayback archive')
        return []
    snapshots = []
    for entry in json_data:
        snapshot_url = parse_snapshot(entry, settings.wayback.api_web, raw=raw) 
        snapshot = Snapshot(snapshot_url, entry[settings.wayback.status_key])
        snapshots.append(snapshot)
    yield snapshots

def main():
    url = sys.argv[1]
    mc = ['200']
    snapshots = fetch_cdx_snapshots(url, 1, mc)
    print(snapshots)

if __name__=='__main__':
    main()
